/*
  ____    ___  ____                    
 |  _ \  / __)|  _ \                   
 | |_) )/ /_  | |_) )                  
 |  __/(   _) |  __/              _    
 | |    \ \__ | |     |\/| /\ |\ |_ |  
 |_|     \___)|_|     |  | \/ |/ |_ |_ 
                                       
*/

/**Parallel Episodic Processing (PEP) Model
 * @version 3.0.0
 * @author James R. Schmidt <james.schmidt@u-bourgogne.fr>
 * Université Bourgogne Franche-Comté*/

package pep;

import java.util.Arrays;
import java.util.concurrent.Semaphore;

/**This is the Main class in which the model variables need to be
 * defined (in the VARIABLES section of the code).*/
public class Main{
 //----------VARIABLES------------------------------------------
 /**DataSet source*/
 static String source = "cueTaskSwitch.txt";
 /**number of subjects*/              static final int nSubjects = 500;
 /**number of concurrent threads*/    public static final int maxThreads = 5;
 /**cycle time resolution*/           public static final int resolution = 10;
 //-------------------------------------------------------------
 /**random with replacement?*/        static boolean isReplace = true;
 /**number of blocks per subject*/    static final int[] nBlocks = {1};
 /**number of no replacement trials*/ static final int[] nSamples = {800};
 /**two alternating stimulus sets?*/  static boolean twoSets = false;
 /**paired trial order?*/             static boolean isPaired = false;
 //-------------------------------------------------------------
 /**number of targets*/               static final int nTargets = 8;
 /**concurrent targets*/              static final int multTarg = 1;
 /**number of distracters*/           static final int nDistracters = 0;
 /**concurrent distracters*/          static final int multDist = 0;
 /**number of cues*/                  static final int nCues = 4;
 /**concurrent cues*/                 static final int multCue = 1;
 /**number of decisions*/             static final int nDecisions = 4;
 /**number of goals*/                 static final int nGoals = 2;
 /**number of responses*/             static final int nResponses = 2;
 /**number conflict stimuli*/         static final int nConflicting = 0;
 /**conditions per variable*/         static final int[] numPerVar = {2,2};
 //-------------------------------------------------------------
 /**target bias*/            static final double tBias = 1.0;
 /**target signal*/          static final double tSignal = 1.0;
 /**distracter bias*/        static final double dBias = 1.0;
 /**distracter signal*/      static final double dSignal = 1.0;
 /**degraded signal*/        static final double tDegrade = .65;
 /**degraded stimuli?*/      static final boolean isDegrade = false;
 //-------------------------------------------------------------
 /**target onset time*/      public static final int tOnset = 201;
 /**target offset time*/     public static final int tOffset = 3200;
 /**distracter onset time*/  public static final int dOnset = 0;
 /**distracter offset time*/ public static final int dOffset = 0;
 /**cue onset time*/         public static final int cOnset = 1;
 /**cue offset time*/        public static final int cOffset = 3200;
 /**distracter mask*/        public static final int dMask = 0;
 /**trial length*/           public static final int trialLength = 3200;
 /**interstimulus interval*/ public static final int isi = 500;
 //-------------------------------------------------------------
 /**trim previous errors?*/  static boolean prevError = true;
 /**check sequence?*/        static boolean isCheckSeq = false;
 /**filter repetitions?*/    static boolean isTrimmed = false;
 /**check all rep types*/    static boolean isAllSeq = false;
 /**cued task switching?*/   static boolean isCueSwitch = true;
 /**only switch cost?*/      static boolean onlySwitch = false;
 /**only cue factor?*/       static boolean onlyCue = false;
 /**response repetition?*/   static boolean onlyResponse = false;
 /**switch plus congruency*/ static boolean andCong = false;
 /**switch only congruency*/ static boolean onlyCong = false;
 //-------------------------------------------------------------
 /**overtrained target-decision connections*/
 static int[][] targDec = {{0,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7},
                           {0,1,0,1,1,0,1,0,2,2,2,2,3,3,3,3}};
 /**overtrained distracter-decision connections*/
 static int[][] distDec = {{},
                           {}};
 /**overtrained decision-response connections*/
 static int[][] knownRes = {{},
                            {}};
 //-------------------------------------------------------------
 /**overtrained goal-decision links*/
 static int[][] goalDec = {{0,0,1,1},
                           {0,1,2,3}};
 //-------------------------------------------------------------
 /**instructed cue-goal connections*/
 static int[][] iCueGoal = {{0,1,2,3},
                            {0,0,1,1}};
 /**instructed stimulus-decision-response connections*/
 static int[][] iStimRes = {{},
                            {}};
 /**instructed decision-response connections*/
 static int[][] iDecRes = {{0,1,2,3},
                           {0,1,0,1}};
 //-------------------------------------------------------------
 /**thread manager*/            public static Semaphore freeThread;
 /**currently active threads*/  public static int queue[];
 /**array of subject threads*/  public static RunTime runs[];
 /**simulation feedback?*/ private static final boolean isVerbose = true;

 /**For each participant, the main method calls RunTime (the guts of the
  * model), and then generates condition means with DataAnalysis and
  * creates a concatenated datafile with Concat.
  * @param args Not used.
  * @throws java.lang.InterruptedException*/
 public static void main(String[] args) throws InterruptedException{
  freeThread = new Semaphore(maxThreads);
  queue = new int[maxThreads];
  runs = new RunTime[nSubjects];
  int maxQueue = (maxThreads<nSubjects) ? maxThreads : nSubjects;
  for(int subj=0; subj<maxQueue; subj++){
   runs[subj] = new RunTime(subj);
   Thread t = new Thread(runs[subj]);
   t.start();
   track(subj);
   queue[subj] = subj; //fill queue first
  }
  freeThread.drainPermits();
  for(int subj=maxThreads; subj<nSubjects; subj++){
   freeThread.acquire();
   for(int i=0; i<maxThreads; i++){
    if(runs[queue[i]].deathRow){
     runs[subj] = new RunTime(subj);
     Thread t = new Thread(runs[subj]);
     t.start();
     track(subj);
     runs[queue[i]] = null; //free stack heap
     queue[i] = subj;
     break;
    }
   }
  }

  freeThread.acquire(maxThreads);
  Arrays.fill(runs,null);
  for(int i=0; i<maxThreads; i++) runs[queue[i]] = null; //free stack heap
  if(isVerbose) System.out.println("Run complete! Processing data...");

  Concat.main(new String[0]);
  if(isVerbose) System.out.println("Data files concatenated...");
  freeThread.release(maxThreads);

  DataAnalysis.main(new String[0]);
  if(isVerbose) System.out.println("Data analysis complete...");
 }

 private static void track(int subj){
  if(isVerbose)
   System.out.println(""+(subj+1)+" of "+nSubjects+" subjects started...");
 }
}